#ifndef DIRECTWALK_HH_
#define DIRECTWALK_HH_

	

#include "ell/State.hh"
#include "ell/StateCollector.hh"

namespace ell {
	
	  /*!
	   * A DirectWalk implements an algorithm to determine a directed path/walk
	   * between two States in the energy landscape. The visited States along
	   * the path are collected using a StateCollector.
	   * 
	   * @author: Hannes Kochni, Martin Mann
	   */
	class DirectWalk {
	public:
		
		//! virtual destructor
			virtual ~DirectWalk() {};
			
		//! Performs a directed walk between two specified States in the 
		//! landscape and stores all visited States.
		//!
		//! @param begin the begin of the walk
		//! @param end the end of the walk
		//! @param scWalk the StateCollector each step of the walk is added to
		//! @return the modified scWalk StateCollector
		virtual 
		StateCollector*
		walk(	const State* const begin
				, const State* const end
				, StateCollector* const scWalk
			) const = 0;
		
	};
	
} // namespace ell


#endif /*DIRECTWALK_HH_*/
