#include "ell/StateCollector.hh"

#include <cmath>

namespace ell
{
	
	SC_PartitionFunction::SC_PartitionFunction()
	  :	SC_Counting(), Z(0.0)
	{
	}
	
	SC_PartitionFunction::~SC_PartitionFunction()
	{
	}

	
	void
	SC_PartitionFunction::
	add(const State& s)
	{
		  // increase the partition function
		Z += exp( - s.getEnergy() / LandscapeTopology::BOLTZMANN_KT );
		  // store the state
		SC_Counting::add(s);
	}

	
	double
	SC_PartitionFunction::
	getZ( void ) const
	{
		return Z;
	}
	
	double
	SC_PartitionFunction::
	getEnergy( void ) const
	{
		return - LandscapeTopology::BOLTZMANN_KT * log( Z );
	}
	

}



namespace ell
{
	
	SC_PartitionFunction_Shifted::
	SC_PartitionFunction_Shifted( const LandscapeTopology& lt )
	  :	SC_Counting()
	  	, Z_shifted(0.0)
	  	, MIN_E( lt.getMFEState()->getEnergy() )
	{
	}
	
	SC_PartitionFunction_Shifted::
	SC_PartitionFunction_Shifted( const double MinE )
	  :	Z_shifted(0.0)
	  	, MIN_E( MinE )
	{
	}
	
	SC_PartitionFunction_Shifted::
	~SC_PartitionFunction_Shifted()
	{
	}

	
	void
	SC_PartitionFunction_Shifted::
	add(const State& s)
	{
		assertbiu(s.getEnergy() >= MIN_E
			, "the used MIN_E is HIGHER than the energy of the collected State");
		  // increase the shifted partition function
		Z_shifted += exp( (MIN_E - s.getEnergy()) 
							/ LandscapeTopology::BOLTZMANN_KT );
		  // store the state
		SC_Counting::add(s);
	}

	
	double
	SC_PartitionFunction_Shifted::
	getZ( void ) const
	{
		return Z_shifted / exp( MIN_E / LandscapeTopology::BOLTZMANN_KT );
	}
	
	double
	SC_PartitionFunction_Shifted::
	getEnergy( void ) const
	{
		return MIN_E - LandscapeTopology::BOLTZMANN_KT * log( Z_shifted );
	}
	

}
