#include "ell/SamplingAbortionCriterion.hh"

namespace ell
{

	////////////////////////////////////////////////////////////////////////////
	SamplingAbortionCriterion::~SamplingAbortionCriterion()
	////////////////////////////////////////////////////////////////////////////
	{
	}

} // namespace ell

namespace ell
{

	////////////////////////////////////////////////////////////////////////////
	SAC_MaxIterations::
	SAC_MaxIterations( const size_t maxIterations )
	 :	MAX_ITERATIONS(maxIterations)
	 	, doneIterations(0)
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	////////////////////////////////////////////////////////////////////////////
	SAC_MaxIterations::~SAC_MaxIterations()
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	
	////////////////////////////////////////////////////////////////////////////
	bool
	SAC_MaxIterations::
	abortSampling(	const LandscapeTopology* const lt
					, const bool ltChanged )
	////////////////////////////////////////////////////////////////////////////
	{
		doneIterations++;
		return doneIterations > MAX_ITERATIONS;
	}

	
	////////////////////////////////////////////////////////////////////////////
	void
	SAC_MaxIterations::
	reset( void )
	////////////////////////////////////////////////////////////////////////////
	{
		doneIterations = 0;
	}

} // namespace ell



namespace ell
{

	////////////////////////////////////////////////////////////////////////////
	SAC_MaxNoChange::
	SAC_MaxNoChange( const size_t maxNoChangeIterations )
	 :	MAX_NO_CHANGE_ITERATIONS(maxNoChangeIterations)
	 	, iterationsWithoutChange(0)
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	////////////////////////////////////////////////////////////////////////////
	SAC_MaxNoChange::~SAC_MaxNoChange()
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	
	////////////////////////////////////////////////////////////////////////////
	bool
	SAC_MaxNoChange::
	abortSampling(	const LandscapeTopology* const lt
					, const bool ltChanged )
	////////////////////////////////////////////////////////////////////////////
	{
		  // check if lt was changed
		if ( ltChanged ) {
			iterationsWithoutChange = 0;
		} else {
			iterationsWithoutChange++;
		}
		return iterationsWithoutChange >= MAX_NO_CHANGE_ITERATIONS;
	}

	
	////////////////////////////////////////////////////////////////////////////
	void
	SAC_MaxNoChange::
	reset( void )
	////////////////////////////////////////////////////////////////////////////
	{
		iterationsWithoutChange = 0;
	}

} // namespace ell

namespace ell
{

	////////////////////////////////////////////////////////////////////////////
	SAC_NoMinimaChange::
	SAC_NoMinimaChange( const size_t maxNoChangeIterations )
	 :	MAX_NO_CHANGE_ITERATIONS(maxNoChangeIterations)
	 	, iterationsWithoutChange(0)
	 	, lastNumOfMinima(0)
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	////////////////////////////////////////////////////////////////////////////
	SAC_NoMinimaChange::~SAC_NoMinimaChange()
	////////////////////////////////////////////////////////////////////////////
	{
	}
	
	
	////////////////////////////////////////////////////////////////////////////
	bool
	SAC_NoMinimaChange::
	abortSampling(	const LandscapeTopology* const lt
					, const bool ltChanged )
	////////////////////////////////////////////////////////////////////////////
	{
		  // check if minima number has changed
		if (!ltChanged || lt->getMinCount() == lastNumOfMinima) {
			iterationsWithoutChange++;
		} else {
			  // minima number has changed --> update counter
			iterationsWithoutChange = 0;
			lastNumOfMinima = lt->getMinCount();
		}
		return iterationsWithoutChange >= MAX_NO_CHANGE_ITERATIONS;
	}

	
	////////////////////////////////////////////////////////////////////////////
	void
	SAC_NoMinimaChange::
	reset( void )
	////////////////////////////////////////////////////////////////////////////
	{
		iterationsWithoutChange = 0;
		lastNumOfMinima = 0;
	}

} // namespace ell


