#ifndef ELL_WAC_LP_HH_
#define ELL_WAC_LP_HH_


#include "ell/StateCollector.hh"
#include "ell/WalkAbortionCriterion.hh"

#include <biu/LatticeDescriptor.hh>

#include <set>

namespace ell
{
	/*!
	 * A WalkAbortionCriterion for S_LP* lattice protein states.
	 * A walk is aborted if the last added state of the given StateCollector
	 * equals the given structure. 
	 * 
	 * @author Martin Mann 
	 */
	class WAC_LP_final: public WalkAbortionCriterion
	{
	protected:
		
		  //! the set of all absolute move strings symmetric to the given 
		  //! structure
		std::set<std::string> absMoveStrings;
		
		  //! the letters of the current move alphabet to derive the move 
		  //! string from the state string representation
		std::string moveAlphabet;
		
	public:
		
		  /*! Construction
		   * 
		   * @param absMoves the protein structure to check for
		   * @param latDescr the underlying lattice model
		   */
		WAC_LP_final(	const std::string& absMoves
						, const biu::LatticeDescriptor& latDescr);
		
		virtual ~WAC_LP_final();
		
		/*! Checks whether or not the last lattice protein state adopts a
		 * certain structure. 
		 * Returns true if the last added state equals the structure, false
		 *               otherwise.
		 */
		virtual bool abort(const StateCollector* const sc) const;
	};
			
} // namespace ell

#endif /*ELL_WAC_LP_HH_*/
